"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPaths;

var _changeCase = require("change-case");

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _getPublicUrlOrPath = _interopRequireDefault(require("./getPublicUrlOrPath"));

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

var _getLocation = _interopRequireDefault(require("./getLocation"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function createPaths(target) {
  const modularRoot = (0, _getModularRoot.default)();
  const targetDirectory = await (0, _getLocation.default)(target);
  const targetName = (0, _changeCase.paramCase)(target); // Make sure any symlinks in the project folder are resolved:
  // https://github.com/facebook/create-react-app/issues/637

  const appDirectory = _fsExtra.default.realpathSync(targetDirectory);

  const resolveApp = relativePath => path.resolve(appDirectory, relativePath);

  // We use `PUBLIC_URL` environment variable or "homepage" field to infer
  // "public path" at which the app is served.
  // webpack needs to know it to put the right <script> hrefs into HTML even in
  // single-page apps that may serve index.html for nested URLs like /todos/42.
  // We can't use a relative path in HTML because we don't want to load something
  // like /todos/42/static/js/bundle.7289d.js. We have to know the root.
  const publicUrlOrPath = (0, _getPublicUrlOrPath.default)(process.env.NODE_ENV === 'development', _fsExtra.default.readJSONSync(resolveApp('package.json')).homepage, process.env.PUBLIC_URL);
  const moduleFileExtensions = ['web.mjs', 'mjs', 'web.js', 'js', 'web.ts', 'ts', 'web.tsx', 'tsx', 'json', 'web.jsx', 'jsx']; // Resolve file paths in the same order as webpack

  const resolveModule = (resolveFn, filePath) => {
    const extension = moduleFileExtensions.find(extension => _fsExtra.default.existsSync(resolveFn(`${filePath}.${extension}`)));

    if (extension) {
      return resolveFn(`${filePath}.${extension}`);
    }

    return resolveFn(`${filePath}.js`);
  };

  const ownPath = path.dirname(path.join(__dirname, '..', '..'));

  const resolveOwn = relativePath => path.resolve(ownPath, relativePath);

  const dotenv = resolveApp('.env');
  const appPath = resolveApp('.');
  const appPublic = resolveApp('public');
  const appHtml = resolveApp('public/index.html');
  const appIndexJs = resolveModule(resolveApp, 'src/index');
  const appPackageJson = resolveApp('package.json');
  const appSrc = resolveApp('src');
  const appTsConfig = resolveApp('tsconfig.json');
  const testsSetup = resolveModule(resolveApp, 'src/setupTests');
  const proxySetup = resolveApp('src/setupProxy.js');
  const appNodeModules = resolveApp('node_modules');
  const ownNodeModules = resolveOwn('node_modules');
  const appTypeDeclarations = resolveApp('src/react-app-env.d.ts');
  const ownTypeDeclarations = resolveOwn('react-app.d.ts');
  const appBuild = path.join(modularRoot, 'dist', targetName);
  const paths = {
    modularRoot,
    publicUrlOrPath,
    dotenv,
    appPath,
    appBuild,
    appPublic,
    appHtml,
    appIndexJs,
    appPackageJson,
    appSrc,
    appTsConfig,
    testsSetup,
    proxySetup,
    appNodeModules,
    ownNodeModules,
    appTypeDeclarations,
    ownTypeDeclarations,
    moduleFileExtensions
  };
  return paths;
}
//# sourceMappingURL=createPaths.js.map