"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _execa = _interopRequireDefault(require("execa"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _isCi = _interopRequireDefault(require("is-ci"));

var _updateNotifier = _interopRequireDefault(require("update-notifier"));

var logger = _interopRequireWildcard(require("./logger"));

var semver = _interopRequireWildcard(require("semver"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function isYarnInstalled() {
  try {
    await (0, _execa.default)('yarnpkg', ['-v']);
    return true;
  } catch (err) {
    return false;
  }
}

async function startupCheck() {
  const {
    name,
    version,
    engines
  } = fs.readJSONSync(require.resolve('modular-scripts/package.json'));
  const nodeEngines = engines === null || engines === void 0 ? void 0 : engines.node;

  if (!semver.satisfies(process.version, nodeEngines)) {
    throw new Error(`${process.version} does not satisfy modular engine constraint ${nodeEngines}`);
  }

  if (process.env.SKIP_MODULAR_STARTUP_CHECK === 'true' || _isCi.default) {
    if (!_isCi.default) {
      logger.warn('Skipping modular startup checks.');
    }
  } else {
    const message = 'modular is out of date - you can update from ' + _chalk.default.dim('{currentVersion}') + _chalk.default.reset(' → ') + _chalk.default.green('{latestVersion}') + ' \nRun yarn add modular-scripts@{latestVersion} to update';
    const notifier = (0, _updateNotifier.default)({
      pkg: {
        name: name,
        version: version
      },
      shouldNotifyInNpmScript: true
    });

    if (notifier.update) {
      notifier.notify({
        message,
        defer: false,
        isGlobal: false
      });
    } else {
      logger.debug(`Running latest version of modular ${version}`);
    }

    if ((await isYarnInstalled()) === false) {
      throw new Error('Please install `yarn` before attempting to run `modular-scripts`.');
    }
  }
}

var _default = startupCheck;
exports.default = _default;
//# sourceMappingURL=startupCheck.js.map